#!/usr/bin/env python

"""
Keyforge Compendium API: https://keyforge-compendium.com/docs
"""

import requests
import json
from requests.auth import HTTPBasicAuth

from Card import Card
from Deck import Deck
from FAQ import FAQ
from Tag import Tag

class KeyPendium:
    """
    Simple Python wrapper for the Keyforge Compendium API.

    >>> KeyforgeCompendium('apikey', 'secret').Cards()

    """
    def __init__(self, key, secret):
        self.baseUrl = "https://keyforge-compendium.com/api/v1/"
        self.cardsUrl = "cards/{}"
        self.setsUrl = "sets/{}/cards/{}"
        self.decksUrlById = "decks/{}"
        self.decksUrlPaged = "decks?page_id={}"
        self.decksUrlByName = "decks/by_name/{}?page_id={}"
        self.decksUrlRandom = "decks/random/{}"
    
        self.key = key
        self.secret = secret

        self.basicAuth = HTTPBasicAuth(self.key, self.secret)
    
    def Cards(self, card_id=None, set_id=None):
        """
        Wrapper around the Cards API endpoint. Retrieves information
        about individual Keyforge cards. Calling this function without
        a card_id but WITH a set_id will return all cards for that set.
        Calling this endpoint without either parameter will return all
        cards in the default Call of the Archons set.

        Args:
            card_id (int) (optional): An individual card's id
            set_id (int) (optional): A set id. As of this writing (7/2/19)
                                     the only valid options are 1 and 2.

        Returns:
            List of Card objects. In the case that a card_id is passed in,
            a list of length one is returned.
        """
        
        if set_id is None:
            if card_id is None:
                return [Card(x) for x in self.MakeRequest(self.cardsUrl, [''])]
            else:
                return [Card(self.MakeRequest(self.cardsUrl, [card_id]))]
        else:
            if card_id is None:
                return [Card(x) for x in self.MakeRequest(self.setsUrl, [set_id, ''])]
            else:
                return [Card(self.MakeRequest(self.setsUrl, [set_id, card_id]))]

    def Decks(self, deck_name=None, page_id=None):
        """
        Wrapper around the Decks API endpoint. Retrieves information
        about publicly registered Keyforge Decks. Calling this endpoint
        without a page_id will return the first 10 pages of
        results. The deck_name parameter allows for name searching and requires
        a query string of at least 4 characters.

        Args:
            deck_name (str) (optional): The deck name search query string
            page_id (int) (optional): The page of results to retrieve. The API
                                      does not at this time provide information
                                      about your current page or the number of total
                                      pages, so you'll have to track these manually

        Returns:
            List of Deck objects.
        """
        results = []
        if page_id is None:
            page_id = 1

        if deck_name is None:
            active_url = self.decksUrlPaged
            params = [page_id]
        else:
            active_url = self.decksUrlByName
            params = [deck_name, page_id]

        response = [Deck(x) for x in self.MakeRequest(active_url, params)]
        results = results + response
        while len(response) > 0 and params[len(params)-1] < 10:
            params[len(params)-1] = params[len(params)-1] + 1
            response = [Deck(x) for x in self.MakeRequest(active_url, params)]
            results = results + response

        return results

    def Deck(self, deck_id=None):
        """
        Wrapper around the Decks API endpoint, specifically the functionality
        to retrieve a single deck by deck UUID. Calling this endpoint without
        a deck_id simply returns the very first public deck.

        Args:
            deck_id (uuid as str) (optional): The UUID for the deck. This can be found within
            the Decks API or on the main Keyforge site for a given deck.

        Returns;
            List containing a single Deck object.
        """
        if deck_id is None:
            return [self.Decks()[0]]
        return [Deck(self.MakeRequest(self.decksUrlById, [deck_id]))]

    def RandomDeck(self, num):
        """
        Wrapper around the Decks API endpoint, specifically the get random
        decks functionality. Note that if a high enough number is passed in this
        response will be paged. Since the values are random anyways, calling this
        function multiple times intead of using pagination is encouraged.

        Args:
            num (int): The number of random decks to return

        Returns:
            List of Deck objects.
        
        """
        if num is None:
            num = 5
        return [Deck(x) for x in self.MakeRequest(self.decksUrlRandom, [num])]

    def MakeRequest(self, url, params):
        """
        Helper function to abstract out the api calls to the Keyforge Compendium API.

        Args:
            url (str): The formatted url (containing {} for data insertion) to query
            params (list): A list of strings to format into the url string

        Returns:
            JSON object of the results
        """
        fullUrl = self.baseUrl + url.format(*params)
        test = json.loads(requests.get(fullUrl, auth=self.basicAuth).content)
        return test
